@@ -19,8 +19,8 @@ For a more genericized version of this example, see
 poly_assoc_generic.py.
 """
 
-from sqlalchemy import *
-from sqlalchemy.orm import *
+from sqlalchemy import MetaData, Table, Column, Integer, String, ForeignKey
+from sqlalchemy.orm import mapper, relation, create_session, class_mapper
 
 metadata = MetaData('sqlite://')
 
@@ -137,11 +137,11 @@ o1.address = Address()
 o1.address.street = '444 park ave.'
 
 sess = create_session()
-sess.save(u1)
-sess.save(o1)
+sess.add(u1)
+sess.add(o1)
 sess.flush()
 
-sess.clear()
+sess.expunge_all()
 
 # query objects, get their addresses
 
