@@ -193,6 +193,14 @@ class PGBoolean(sqltypes.Boolean):
     def get_col_spec(self):
         return "BOOLEAN"
 
+class PGBit(sqltypes.TypeEngine):
+    def get_col_spec(self):
+        return "BIT"
+        
+class PGUuid(sqltypes.TypeEngine):
+    def get_col_spec(self):
+        return "UUID"
+    
 class PGArray(sqltypes.MutableType, sqltypes.Concatenable, sqltypes.TypeEngine):
     def __init__(self, item_type, mutable=True):
         if isinstance(item_type, type):
@@ -283,6 +291,8 @@ ischema_names = {
     'real' : PGFloat,
     'inet': PGInet,
     'cidr': PGCidr,
+    'uuid':PGUuid,
+    'bit':PGBit,
     'macaddr': PGMacAddr,
     'double precision' : PGFloat,
     'timestamp' : PGDateTime,
@@ -527,6 +537,7 @@ class PGDialect(default.DefaultDialect):
             elif attype == 'timestamp without time zone':
                 kwargs['timezone'] = False
 
+            coltype = None
             if attype in ischema_names:
                 coltype = ischema_names[attype]
             else:
@@ -540,8 +551,6 @@ class PGDialect(default.DefaultDialect):
                             # It can, however, override the default value, but can't set it to null.
                             default = domain['default']
                         coltype = ischema_names[domain['attype']]
-                else:
-                    coltype = None
 
             if coltype:
                 coltype = coltype(*args, **kwargs)
@@ -638,10 +647,11 @@ class PGDialect(default.DefaultDialect):
         for row in c.fetchall():
             idx_name, unique, expr, prd, col = row
 
-            if expr and not idx_name == sv_idx_name:
-                util.warn(
-                  "Skipped unsupported reflection of expression-based index %s"
-                  % idx_name)
+            if expr:
+                if not idx_name == sv_idx_name:
+                    util.warn(
+                      "Skipped unsupported reflection of expression-based index %s"
+                      % idx_name)
                 sv_idx_name = idx_name
                 continue
             if prd and not idx_name == sv_idx_name:
@@ -782,6 +792,12 @@ class PGCompiler(compiler.DefaultCompiler):
         else:
             return text
 
+    def visit_extract(self, extract, **kwargs):
+        field = self.extract_map.get(extract.field, extract.field)
+        return "EXTRACT(%s FROM %s::timestamp)" % (
+            field, self.process(extract.expr))
+
+
 class PGSchemaGenerator(compiler.SchemaGenerator):
     def get_column_specification(self, column, **kwargs):
         colspec = self.preparer.format_column(column)
