@@ -115,8 +115,8 @@ class ColumnLoader(LoaderStrategy):
         if adapter:
             col = adapter.columns[col]
         if col in row:
-            def new_execute(state, row, **flags):
-                state.dict[key] = row[col]
+            def new_execute(state, dict_, row, **flags):
+                dict_[key] = row[col]
                 
             if self._should_log_debug:
                 new_execute = self.debug_callable(new_execute, self.logger,
@@ -125,7 +125,7 @@ class ColumnLoader(LoaderStrategy):
                 )
             return (new_execute, None)
         else:
-            def new_execute(state, row, isnew, **flags):
+            def new_execute(state, dict_, row, isnew, **flags):
                 if isnew:
                     state.expire_attributes([key])
             if self._should_log_debug:
@@ -171,15 +171,15 @@ class CompositeColumnLoader(ColumnLoader):
             columns = [adapter.columns[c] for c in columns]
         for c in columns:
             if c not in row:
-                def new_execute(state, row, isnew, **flags):
+                def new_execute(state, dict_, row, isnew, **flags):
                     if isnew:
                         state.expire_attributes([key])
                 if self._should_log_debug:
                     self.logger.debug("%s deferring load" % self)
                 return (new_execute, None)
         else:
-            def new_execute(state, row, **flags):
-                state.dict[key] = composite_class(*[row[c] for c in columns])
+            def new_execute(state, dict_, row, **flags):
+                dict_[key] = composite_class(*[row[c] for c in columns])
 
             if self._should_log_debug:
                 new_execute = self.debug_callable(new_execute, self.logger,
@@ -201,12 +201,14 @@ class DeferredColumnLoader(LoaderStrategy):
         if col in row:
             return self.parent_property._get_strategy(ColumnLoader).create_row_processor(selectcontext, path, mapper, row, adapter)
 
-        elif not self.is_class_level or len(selectcontext.options):
-            def new_execute(state, row, **flags):
-                state.set_callable(self.key, self.setup_loader(state))
+        elif not self.is_class_level:
+            def new_execute(state, dict_, row, **flags):
+                state.set_callable(self.key, LoadDeferredColumns(state, self.key))
         else:
-            def new_execute(state, row, **flags):
-                state.reset(self.key)
+            def new_execute(state, dict_, row, **flags):
+                # reset state on the key so that deferred callables
+                # fire off on next access.
+                state.reset(self.key, dict_)
 
         if self._should_log_debug:
             new_execute = self.debug_callable(new_execute, self.logger, None,
@@ -227,7 +229,7 @@ class DeferredColumnLoader(LoaderStrategy):
              compare_function=self.columns[0].type.compare_values,
              copy_function=self.columns[0].type.copy_value,
              mutable_scalars=self.columns[0].type.is_mutable(),
-             callable_=self.class_level_loader,
+             callable_=self._class_level_loader,
              dont_expire_missing=True
         )
 
@@ -238,46 +240,31 @@ class DeferredColumnLoader(LoaderStrategy):
             
             self.parent_property._get_strategy(ColumnLoader).setup_query(context, entity, path, adapter, **kwargs)
     
-    def class_level_loader(self, state, props=None):
+    def _class_level_loader(self, state):
         if not mapperutil._state_has_identity(state):
             return None
             
-        localparent = mapper._state_mapper(state)
-
-        # adjust for the ColumnProperty associated with the instance
-        # not being our own ColumnProperty.  
-        # TODO: this may no longer be relevant without entity_name.
-        prop = localparent.get_property(self.key)
-        if prop is not self.parent_property:
-            return prop._get_strategy(DeferredColumnLoader).setup_loader(state)
-
-        return LoadDeferredColumns(state, self.key, props)
+        return LoadDeferredColumns(state, self.key)
         
-    def setup_loader(self, state, props=None, create_statement=None):
-        return LoadDeferredColumns(state, self.key, props)
                 
 log.class_logger(DeferredColumnLoader)
 
 class LoadDeferredColumns(object):
     """serializable loader object used by DeferredColumnLoader"""
     
-    def __init__(self, *args):
-        self.state, self.key, self.keys = args
+    def __init__(self, state, key):
+        self.state, self.key = state, key
 
     def __call__(self):
         state = self.state
         
-        if not mapper._state_has_identity(state):
-            return None
         
         localparent = mapper._state_mapper(state)
         
         prop = localparent.get_property(self.key)
         strategy = prop._get_strategy(DeferredColumnLoader)
 
-        if self.keys:
-            toload = self.keys
-        elif strategy.group:
+        if strategy.group:
             toload = [
                     p.key for p in 
                     localparent.iterate_properties 
@@ -292,11 +279,18 @@ class LoadDeferredColumns(object):
         group = [k for k in toload if k in state.unmodified]
 
         if strategy._should_log_debug:
-            strategy.logger.debug("deferred load %s group %s" % (mapperutil.state_attribute_str(state, self.key), group and ','.join(group) or 'None'))
+            strategy.logger.debug(
+                    "deferred load %s group %s" % 
+                    (mapperutil.state_attribute_str(state, self.key), group and ','.join(group) or 'None')
+                )
 
         session = sessionlib._state_session(state)
         if session is None:
-            raise sa_exc.UnboundExecutionError("Parent instance %s is not bound to a Session; deferred load operation of attribute '%s' cannot proceed" % (mapperutil.state_str(state), self.key))
+            raise sa_exc.UnboundExecutionError(
+                        "Parent instance %s is not bound to a Session; "
+                        "deferred load operation of attribute '%s' cannot proceed" % 
+                        (mapperutil.state_str(state), self.key)
+                    )
 
         query = session.query(localparent)
         ident = state.key[1]
@@ -346,7 +340,7 @@ class NoLoader(AbstractRelationLoader):
         )
 
     def create_row_processor(self, selectcontext, path, mapper, row, adapter):
-        def new_execute(state, row, **flags):
+        def new_execute(state, dict_, row, **flags):
             self._init_instance_attribute(state)
 
         if self._should_log_debug:
@@ -380,7 +374,7 @@ class LazyLoader(AbstractRelationLoader):
         _register_attribute(self, 
                 mapper,
                 useobject=True,
-                callable_=self.class_level_loader,
+                callable_=self._class_level_loader,
                 uselist = self.parent_property.uselist,
                 typecallable = self.parent_property.collection_class,
                 )
@@ -435,31 +429,20 @@ class LazyLoader(AbstractRelationLoader):
             criterion = adapt_source(criterion)
         return criterion
         
-    def class_level_loader(self, state, options=None, path=None):
+    def _class_level_loader(self, state):
         if not mapperutil._state_has_identity(state):
             return None
 
-        localparent = mapper._state_mapper(state)
-
-        # adjust for the PropertyLoader associated with the instance
-        # not being our own PropertyLoader.  
-        # TODO: this may no longer be relevant without entity_name
-        prop = localparent.get_property(self.key)
-        if prop is not self.parent_property:
-            return prop._get_strategy(LazyLoader).setup_loader(state)
-        
-        return LoadLazyAttribute(state, self.key, options, path)
-
-    def setup_loader(self, state, options=None, path=None):
-        return LoadLazyAttribute(state, self.key, options, path)
+        return LoadLazyAttribute(state, self.key)
 
     def create_row_processor(self, selectcontext, path, mapper, row, adapter):
-        if not self.is_class_level or len(selectcontext.options):
-            path = path + (self.key,)
-            def new_execute(state, row, **flags):
+        if not self.is_class_level:
+            def new_execute(state, dict_, row, **flags):
                 # we are not the primary manager for this attribute on this class - set up a per-instance lazyloader,
-                # which will override the class-level behavior
-                self._init_instance_attribute(state, callable_=self.setup_loader(state, selectcontext.options, selectcontext.query._current_path + path))
+                # which will override the class-level behavior.
+                # this currently only happens when using a "lazyload" option on a "no load" attribute -
+                # "eager" attributes always have a class-level lazyloader installed.
+                self._init_instance_attribute(state, callable_=LoadLazyAttribute(state, self.key))
 
             if self._should_log_debug:
                 new_execute = self.debug_callable(new_execute, self.logger, None,
@@ -468,12 +451,11 @@ class LazyLoader(AbstractRelationLoader):
 
             return (new_execute, None)
         else:
-            def new_execute(state, row, **flags):
+            def new_execute(state, dict_, row, **flags):
                 # we are the primary manager for this attribute on this class - reset its per-instance attribute state, 
                 # so that the class-level lazy loader is executed when next referenced on this instance.
-                # this usually is not needed unless the constructor of the object referenced the attribute before we got 
-                # to load data into it.
-                state.reset(self.key)
+                # this is needed in populate_existing() types of scenarios to reset any existing state.
+                state.reset(self.key, dict_)
 
             if self._should_log_debug:
                 new_execute = self.debug_callable(new_execute, self.logger, None,
@@ -481,7 +463,7 @@ class LazyLoader(AbstractRelationLoader):
                 )
 
             return (new_execute, None)
-
+            
     def _create_lazy_clause(cls, prop, reverse_direction=False):
         binds = util.column_dict()
         lookup = util.column_dict()
@@ -529,20 +511,17 @@ log.class_logger(LazyLoader)
 class LoadLazyAttribute(object):
     """serializable loader object used by LazyLoader"""
 
-    def __init__(self, *args):
-        self.state, self.key, self.options, self.path = args
+    def __init__(self, state, key):
+        self.state, self.key = state, key
         
     def __getstate__(self):
-        return (self.state, self.key, self.options, serialize_path(self.path))
+        return (self.state, self.key)
 
     def __setstate__(self, state):
-        self.state, self.key, self.options, path = state
-        self.path = deserialize_path(path)
+        self.state, self.key = state
         
     def __call__(self):
         state = self.state
-        if not mapper._state_has_identity(state):
-            return None
 
         instance_mapper = mapper._state_mapper(state)
         prop = instance_mapper.get_property(self.key)
@@ -561,8 +540,8 @@ class LoadLazyAttribute(object):
         
         q = session.query(prop.mapper)._adapt_all_clauses()
         
-        if self.path:
-            q = q._with_current_path(self.path)
+        if state.load_path:
+            q = q._with_current_path(state.load_path + (self.key,))
             
         # if we have a simple primary key load, use mapper.get()
         # to possibly save a DB round trip
@@ -575,15 +554,15 @@ class LoadLazyAttribute(object):
                 ident.append(val)
             if allnulls:
                 return None
-            if self.options:
-                q = q._conditional_options(*self.options)
+            if state.load_options:
+                q = q._conditional_options(*state.load_options)
             return q.get(ident)
 
         if prop.order_by:
             q = q.order_by(*util.to_list(prop.order_by))
 
-        if self.options:
-            q = q._conditional_options(*self.options)
+        if state.load_options:
+            q = q._conditional_options(*state.load_options)
         q = q.filter(strategy.lazy_clause(state))
 
         result = q.all()
@@ -660,18 +639,26 @@ class EagerLoader(AbstractRelationLoader):
 
         if entity in context.eager_joins:
             entity_key, default_towrap = entity, entity.selectable
-        elif should_nest_selectable or not context.from_clause or not sql_util.search(context.from_clause, entity.selectable):
-            # if no from_clause, or a from_clause we can't join to, or a subquery is going to be generated, 
+
+        elif should_nest_selectable or not context.from_clause:
+            # if no from_clause, or a subquery is going to be generated, 
             # store eager joins per _MappedEntity; Query._compile_context will 
             # add them as separate selectables to the select(), or splice them together
             # after the subquery is generated
             entity_key, default_towrap = entity, entity.selectable
         else:
-            # otherwise, create a single eager join from the from clause.  
-            # Query._compile_context will adapt as needed and append to the
-            # FROM clause of the select().
-            entity_key, default_towrap = None, context.from_clause  
-
+            index, clause = sql_util.find_join_source(context.from_clause, entity.selectable)
+            if clause:
+                # join to an existing FROM clause on the query.
+                # key it to its list index in the eager_joins dict.
+                # Query._compile_context will adapt as needed and append to the
+                # FROM clause of the select().
+                entity_key, default_towrap = index, clause
+            else:
+                # if no from_clause to join to,
+                # store eager joins per _MappedEntity
+                entity_key, default_towrap = entity, entity.selectable
+                
         towrap = context.eager_joins.setdefault(entity_key, default_towrap)
 
         # create AliasedClauses object to build up the eager query.  
@@ -748,24 +735,24 @@ class EagerLoader(AbstractRelationLoader):
             _instance = self.mapper._instance_processor(context, path + (self.mapper.base_mapper,), eager_adapter)
             
             if not self.uselist:
-                def execute(state, row, isnew, **flags):
+                def execute(state, dict_, row, isnew, **flags):
                     if isnew:
                         # set a scalar object instance directly on the
                         # parent object, bypassing InstrumentedAttribute
                         # event handlers.
-                        state.dict[key] = _instance(row, None)
+                        dict_[key] = _instance(row, None)
                     else:
                         # call _instance on the row, even though the object has been created,
                         # so that we further descend into properties
                         _instance(row, None)
             else:
-                def execute(state, row, isnew, **flags):
+                def execute(state, dict_, row, isnew, **flags):
                     if isnew or (state, key) not in context.attributes:
                         # appender_key can be absent from context.attributes with isnew=False
                         # when self-referential eager loading is used; the same instance may be present
                         # in two distinct sets of result columns
 
-                        collection = attributes.init_collection(state, key)
+                        collection = attributes.init_state_collection(state, dict_, key)
                         appender = util.UniqueAppender(collection, 'append_without_event')
 
                         context.attributes[(state, key)] = appender
