@@ -1,5 +1,5 @@
-import testbase
-import unittest
+import testenv; testenv.configure_for_tests()
+from testlib import sa_unittest as unittest
 
 def suite():
     modules_to_test = (
@@ -18,4 +18,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    testbase.main(suite())
+    testenv.main(suite())
