@@ -1,8 +1,13 @@
+# coding: utf-8
+
 import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
 from sqlalchemy.sql import table, column
 from sqlalchemy.databases import oracle
 from testlib import *
+from testlib.testing import eq_
+from testlib.engines import testing_engine
+import os
 
 
 class OutParamTest(TestBase, AssertsExecutionResults):
@@ -10,19 +15,19 @@ class OutParamTest(TestBase, AssertsExecutionResults):
 
     def setUpAll(self):
         testing.db.execute("""
-create or replace procedure foo(x_in IN number, x_out OUT number, y_out OUT number) IS
+create or replace procedure foo(x_in IN number, x_out OUT number, y_out OUT number, z_out OUT varchar) IS
   retval number;
     begin
     retval := 6;
     x_out := 10;
     y_out := x_in * 15;
+    z_out := NULL;
     end;
         """)
 
     def test_out_params(self):
-        result = testing.db.execute(text("begin foo(:x, :y, :z); end;", bindparams=[bindparam('x', Numeric), outparam('y', Numeric), outparam('z', Numeric)]), x=5)
-        assert result.out_parameters == {'y':10, 'z':75}, result.out_parameters
-        print result.out_parameters
+        result = testing.db.execute(text("begin foo(:x_in, :x_out, :y_out, :z_out); end;", bindparams=[bindparam('x_in', Numeric), outparam('x_out', Numeric), outparam('y_out', Numeric), outparam('z_out', String)]), x_in=5)
+        assert result.out_parameters == {'x_out':10, 'y_out':75, 'z_out':None}, result.out_parameters
 
     def tearDownAll(self):
          testing.db.execute("DROP PROCEDURE foo")
@@ -58,8 +63,9 @@ class CompileTest(TestBase, AssertsCompiledSQL):
         
         s = select([t]).limit(10).offset(20)
 
-        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT sometable.col1 AS col1, sometable.col2 AS col2, "
-            "ROW_NUMBER() OVER (ORDER BY sometable.rowid) AS ora_rn FROM sometable) WHERE ora_rn>20 AND ora_rn<=30"
+        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT col1, col2, ROWNUM AS ora_rn "
+            "FROM (SELECT sometable.col1 AS col1, sometable.col2 AS col2 "
+            "FROM sometable) WHERE ROWNUM <= :ROWNUM_1) WHERE ora_rn > :ora_rn_1"
         )
 
         # assert that despite the subquery, the columns from the table,
@@ -69,17 +75,16 @@ class CompileTest(TestBase, AssertsCompiledSQL):
         
         s = select([s.c.col1, s.c.col2])
 
-        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT col1, col2 FROM (SELECT sometable.col1 AS col1, "
-            "sometable.col2 AS col2, ROW_NUMBER() OVER (ORDER BY sometable.rowid) AS ora_rn FROM sometable) WHERE ora_rn>20 AND ora_rn<=30)")
+        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT col1, col2 FROM (SELECT col1, col2, ROWNUM AS ora_rn FROM (SELECT sometable.col1 AS col1, sometable.col2 AS col2 FROM sometable) WHERE ROWNUM <= :ROWNUM_1) WHERE ora_rn > :ora_rn_1)")
 
         # testing this twice to ensure oracle doesn't modify the original statement
-        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT col1, col2 FROM (SELECT sometable.col1 AS col1, "
-            "sometable.col2 AS col2, ROW_NUMBER() OVER (ORDER BY sometable.rowid) AS ora_rn FROM sometable) WHERE ora_rn>20 AND ora_rn<=30)")
+        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT col1, col2 FROM (SELECT col1, col2, ROWNUM AS ora_rn FROM (SELECT sometable.col1 AS col1, sometable.col2 AS col2 FROM sometable) WHERE ROWNUM <= :ROWNUM_1) WHERE ora_rn > :ora_rn_1)")
 
         s = select([t]).limit(10).offset(20).order_by(t.c.col2)
 
-        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT sometable.col1 AS col1, "
-            "sometable.col2 AS col2, ROW_NUMBER() OVER (ORDER BY sometable.col2) AS ora_rn FROM sometable) WHERE ora_rn>20 AND ora_rn<=30")
+        self.assert_compile(s, "SELECT col1, col2 FROM (SELECT col1, col2, ROWNUM "
+            "AS ora_rn FROM (SELECT sometable.col1 AS col1, sometable.col2 AS col2 FROM sometable "
+            "ORDER BY sometable.col2) WHERE ROWNUM <= :ROWNUM_1) WHERE ora_rn > :ora_rn_1")
 
     def test_outer_join(self):
         table1 = table('mytable',
@@ -120,19 +125,24 @@ AND mytable.myid = myothertable.otherid(+)",
 
         query = table1.outerjoin(table2, table1.c.myid==table2.c.otherid).outerjoin(table3, table3.c.userid==table2.c.otherid)
         self.assert_compile(query.select(), "SELECT mytable.myid, mytable.name, mytable.description, myothertable.otherid, myothertable.othername, thirdtable.userid, thirdtable.otherstuff FROM mytable LEFT OUTER JOIN myothertable ON mytable.myid = myothertable.otherid LEFT OUTER JOIN thirdtable ON thirdtable.userid = myothertable.otherid")
-        self.assert_compile(query.select(), "SELECT mytable.myid, mytable.name, mytable.description, myothertable.otherid, myothertable.othername, thirdtable.userid, thirdtable.otherstuff FROM mytable, myothertable, thirdtable WHERE mytable.myid = myothertable.otherid(+) AND thirdtable.userid(+) = myothertable.otherid", dialect=oracle.dialect(use_ansi=False))
+        self.assert_compile(query.select(), "SELECT mytable.myid, mytable.name, mytable.description, myothertable.otherid, myothertable.othername, thirdtable.userid, thirdtable.otherstuff FROM mytable, myothertable, thirdtable WHERE thirdtable.userid(+) = myothertable.otherid AND mytable.myid = myothertable.otherid(+)", dialect=oracle.dialect(use_ansi=False))
 
         query = table1.join(table2, table1.c.myid==table2.c.otherid).join(table3, table3.c.userid==table2.c.otherid)
-        self.assert_compile(query.select(), "SELECT mytable.myid, mytable.name, mytable.description, myothertable.otherid, myothertable.othername, thirdtable.userid, thirdtable.otherstuff FROM mytable, myothertable, thirdtable WHERE mytable.myid = myothertable.otherid AND thirdtable.userid = myothertable.otherid", dialect=oracle.dialect(use_ansi=False))
+        self.assert_compile(query.select(), "SELECT mytable.myid, mytable.name, mytable.description, myothertable.otherid, myothertable.othername, thirdtable.userid, thirdtable.otherstuff FROM mytable, myothertable, thirdtable WHERE thirdtable.userid = myothertable.otherid AND mytable.myid = myothertable.otherid", dialect=oracle.dialect(use_ansi=False))
 
         query = table1.join(table2, table1.c.myid==table2.c.otherid).outerjoin(table3, table3.c.userid==table2.c.otherid)
-        self.assert_compile(query.select().order_by(table1.oid_column).limit(10).offset(5), "SELECT myid, name, description, otherid, othername, userid, \
-otherstuff FROM (SELECT mytable.myid AS myid, mytable.name AS name, \
-mytable.description AS description, myothertable.otherid AS otherid, \
-myothertable.othername AS othername, thirdtable.userid AS userid, \
-thirdtable.otherstuff AS otherstuff, ROW_NUMBER() OVER (ORDER BY mytable.rowid) AS ora_rn \
-FROM mytable, myothertable, thirdtable WHERE mytable.myid = myothertable.otherid AND thirdtable.userid(+) = myothertable.otherid) \
-WHERE ora_rn>5 AND ora_rn<=15", dialect=oracle.dialect(use_ansi=False))
+
+        self.assert_compile(query.select().order_by(table1.c.name).limit(10).offset(5), 
+        
+            "SELECT myid, name, description, otherid, othername, userid, "
+            "otherstuff FROM (SELECT myid, name, description, "
+            "otherid, othername, userid, otherstuff, ROWNUM AS ora_rn FROM (SELECT "
+            "mytable.myid AS myid, mytable.name AS name, mytable.description AS description, "
+            "myothertable.otherid AS otherid, myothertable.othername AS othername, "
+            "thirdtable.userid AS userid, thirdtable.otherstuff AS otherstuff FROM mytable, "
+            "myothertable, thirdtable WHERE thirdtable.userid(+) = myothertable.otherid AND "
+            "mytable.myid = myothertable.otherid ORDER BY mytable.name) WHERE "
+            "ROWNUM <= :ROWNUM_1) WHERE ora_rn > :ora_rn_1", dialect=oracle.dialect(use_ansi=False))
 
     def test_alias_outer_join(self):
         address_types = table('address_types',
@@ -150,13 +160,13 @@ WHERE ora_rn>5 AND ora_rn<=15", dialect=oracle.dialect(use_ansi=False))
         s = select([at_alias, addresses]).\
             select_from(addresses.outerjoin(at_alias, addresses.c.address_type_id==at_alias.c.id)).\
             where(addresses.c.user_id==7).\
-            order_by(addresses.oid_column, address_types.oid_column)
+            order_by(addresses.c.id, address_types.c.id)
         self.assert_compile(s, "SELECT address_types_1.id, address_types_1.name, addresses.id, addresses.user_id, "
             "addresses.address_type_id, addresses.email_address FROM addresses LEFT OUTER JOIN address_types address_types_1 "
-            "ON addresses.address_type_id = address_types_1.id WHERE addresses.user_id = :user_id_1 ORDER BY addresses.rowid, "
-            "address_types.rowid")
+            "ON addresses.address_type_id = address_types_1.id WHERE addresses.user_id = :user_id_1 ORDER BY addresses.id, "
+            "address_types.id")
 
-class SchemaReflectionTest(TestBase, AssertsCompiledSQL):
+class MultiSchemaTest(TestBase, AssertsCompiledSQL):
     """instructions:
 
        1. create a user 'ed' in the oracle database.
@@ -174,6 +184,44 @@ class SchemaReflectionTest(TestBase, AssertsCompiledSQL):
 
     __only_on__ = 'oracle'
 
+    def test_create_same_names_explicit_schema(self):
+        schema = testing.db.dialect.get_default_schema_name(testing.db.connect())
+        meta = MetaData(testing.db)
+        parent = Table('parent', meta, 
+            Column('pid', Integer, primary_key=True),
+            schema=schema
+        )
+        child = Table('child', meta, 
+            Column('cid', Integer, primary_key=True),
+            Column('pid', Integer, ForeignKey('scott.parent.pid')),
+            schema=schema
+        )
+        meta.create_all()
+        try:
+            parent.insert().execute({'pid':1})
+            child.insert().execute({'cid':1, 'pid':1})
+            self.assertEquals(child.select().execute().fetchall(), [(1, 1)])
+        finally:
+            meta.drop_all()
+
+    def test_create_same_names_implicit_schema(self):
+        meta = MetaData(testing.db)
+        parent = Table('parent', meta, 
+            Column('pid', Integer, primary_key=True),
+        )
+        child = Table('child', meta, 
+            Column('cid', Integer, primary_key=True),
+            Column('pid', Integer, ForeignKey('parent.pid')),
+        )
+        meta.create_all()
+        try:
+            parent.insert().execute({'pid':1})
+            child.insert().execute({'cid':1, 'pid':1})
+            self.assertEquals(child.select().execute().fetchall(), [(1, 1)])
+        finally:
+            meta.drop_all()
+
+
     def test_reflect_alt_owner_explicit(self):
         meta = MetaData(testing.db)
         parent = Table('parent', meta, autoload=True, schema='ed')
@@ -250,6 +298,22 @@ class TypesTest(TestBase, AssertsCompiledSQL):
             )
         [[row[k] for k in row.keys()] for row in types_table.select().execute().fetchall()]
 
+    def test_reflect_nvarchar(self):
+        metadata = MetaData(testing.db)
+        t = Table('t', metadata,
+            Column('data', oracle.OracleNVarchar(255))
+        )
+        metadata.create_all()
+        try:
+            m2 = MetaData(testing.db)
+            t2 = Table('t', m2, autoload=True)
+            assert isinstance(t2.c.data.type, oracle.OracleNVarchar)
+            data = u'm’a réveillé.'
+            t2.insert().execute(data=data)
+            eq_(t2.select().execute().fetchone()['data'], data)
+        finally:
+            metadata.drop_all()
+        
     def test_longstring(self):
         metadata = MetaData(testing.db)
         testing.db.execute("""
@@ -266,6 +330,52 @@ class TypesTest(TestBase, AssertsCompiledSQL):
         finally:
             testing.db.execute("DROP TABLE Z_TEST")
 
+    def test_raw_lobs(self):
+        engine = testing_engine(options=dict(auto_convert_lobs=False))
+        metadata = MetaData()
+        t = Table("z_test", metadata, Column('id', Integer, primary_key=True), 
+                 Column('data', Text), Column('bindata', Binary))
+        t.create(engine)
+        try:
+            engine.execute(t.insert(), id=1, data='this is text', bindata='this is binary')
+            row = engine.execute(t.select()).fetchone()
+            eq_(row['data'].read(), 'this is text')
+            eq_(row['bindata'].read(), 'this is binary')
+        finally:
+            t.drop(engine)
+class BufferedColumnTest(TestBase, AssertsCompiledSQL):
+    __only_on__ = 'oracle'
+
+    def setUpAll(self):
+        global binary_table, stream, meta
+        meta = MetaData(testing.db)
+        binary_table = Table('binary_table', meta, 
+           Column('id', Integer, primary_key=True),
+           Column('data', Binary)
+        )
+        meta.create_all()
+        stream = os.path.join(os.path.dirname(testenv.__file__), 'binary_data_one.dat')
+        stream = file(stream).read(12000)
+
+        for i in range(1, 11):
+            binary_table.insert().execute(id=i, data=stream)
+
+    def tearDownAll(self):
+        meta.drop_all()
+
+    def test_fetch(self):
+        self.assertEquals(
+            binary_table.select().execute().fetchall() ,
+            [(i, stream) for i in range(1, 11)], 
+        )
+
+    def test_fetch_single_arraysize(self):
+        eng = testing_engine(options={'arraysize':1})
+        self.assertEquals(
+            eng.execute(binary_table.select()).fetchall(),
+            [(i, stream) for i in range(1, 11)], 
+        )
+
 class SequenceTest(TestBase, AssertsCompiledSQL):
     def test_basic(self):
         seq = Sequence("my_seq_no_schema")
@@ -278,6 +388,10 @@ class SequenceTest(TestBase, AssertsCompiledSQL):
         seq = Sequence("My_Seq", schema="Some_Schema")
         assert dialect.identifier_preparer.format_sequence(seq) == '"Some_Schema"."My_Seq"'
 
+class ExecuteTest(TestBase):
+    __only_on__ = 'oracle'
+    def test_basic(self):
+        assert testing.db.execute("/*+ this is a comment */ SELECT 1 FROM DUAL").fetchall() == [(1,)]
 
 if __name__ == '__main__':
     testenv.main()
