@@ -22,6 +22,7 @@ def suite():
         'orm.assorted_eager',
 
         'orm.naturalpks',
+        'orm.defaults',
         'orm.unitofwork',
         'orm.session',
         'orm.transaction',
@@ -40,6 +41,8 @@ def suite():
         'orm.onetoone',
         'orm.dynamic',
 
+        'orm.evaluator',
+
         'orm.deprecations',
         )
     alltests = unittest.TestSuite()
