@@ -6,6 +6,8 @@ from sqlalchemy.orm.session import _sessions
 import operator
 from testlib import testing
 from testlib.sa import MetaData, Table, Column, Integer, String, ForeignKey, PickleType
+import sqlalchemy as sa
+from sqlalchemy.sql import column
 from orm import _base
 
 
@@ -386,6 +388,15 @@ class MemUsageTest(EnsureZeroed):
             go()
         finally:
             metadata.drop_all()
+
+    def test_type_compile(self):
+        from sqlalchemy.databases.sqlite import SQLiteDialect
+        cast = sa.cast(column('x'), sa.Integer)
+        @profile_memory
+        def go():
+            dialect = SQLiteDialect()
+            cast.compile(dialect=dialect)
+        go()
         
 if __name__ == '__main__':
     testenv.main()
