@@ -1,4 +1,3 @@
-import testbase
 import optparse, os, sys, re, ConfigParser, StringIO, time, warnings
 logging, require = None, None
 
@@ -10,6 +9,7 @@ db_label, db_url, db_opts = None, None, {}
 
 options = None
 file_config = None
+coverage_enabled = False
 
 base_config = """
 [db]
@@ -44,6 +44,29 @@ def configure():
 
     return options, file_config
 
+def configure_defaults():
+    global options, config
+    global getopts_options, file_config
+    global db
+
+    file_config = ConfigParser.ConfigParser()
+    file_config.readfp(StringIO.StringIO(base_config))
+    file_config.read(['test.cfg', os.path.expanduser('~/.satest.cfg')])
+    (options, args) = parser.parse_args([])
+
+    # make error messages raised by decorators that depend on a default
+    # database clearer.
+    class _engine_bomb(object):
+        def __getattr__(self, key):
+            raise RuntimeError('No default engine available, testlib '
+                               'was configured with defaults only.')
+
+    db = _engine_bomb()
+    import testlib.testing
+    testlib.testing.db = db
+
+    return options, file_config
+
 def _log(option, opt_str, value, parser):
     global logging
     if not logging:
@@ -55,24 +78,50 @@ def _log(option, opt_str, value, parser):
     elif opt_str.endswith('-debug'):
         logging.getLogger(value).setLevel(logging.DEBUG)
 
-def _start_coverage(option, opt_str, value, parser):
+def _start_cumulative_coverage(option, opt_str, value, parser):
+    _start_coverage(option, opt_str, value, parser, erase=False)
+
+def _start_coverage(option, opt_str, value, parser, erase=True):
     import sys, atexit, coverage
     true_out = sys.stdout
-
-    def _iter_covered_files():
-        import sqlalchemy
-        for rec in os.walk(os.path.dirname(sqlalchemy.__file__)):
+    
+    global coverage_enabled
+    coverage_enabled = True
+    
+    def _iter_covered_files(mod, recursive=True):
+        
+        if recursive:
+            ff = os.walk
+        else:
+            ff = os.listdir
+            
+        for rec in ff(os.path.dirname(mod.__file__)):
             for x in rec[2]:
                 if x.endswith('.py'):
                     yield os.path.join(rec[0], x)
+            
     def _stop():
         coverage.stop()
         true_out.write("\nPreparing coverage report...\n")
-        coverage.report(list(_iter_covered_files()),
-                        show_missing=False, ignore_errors=False,
-                        file=true_out)
+
+        from sqlalchemy import sql, orm, engine, \
+                            ext, databases, log
+                        
+        import sqlalchemy
+        
+        for modset in [
+            _iter_covered_files(sqlalchemy, recursive=False),
+            _iter_covered_files(databases),
+            _iter_covered_files(engine),
+            _iter_covered_files(ext),
+            _iter_covered_files(orm),
+        ]:
+            coverage.report(list(modset),
+                            show_missing=False, ignore_errors=False,
+                            file=true_out)
     atexit.register(_stop)
-    coverage.erase()
+    if erase:
+        coverage.erase()
     coverage.start()
 
 def _list_dbs(*args):
@@ -129,6 +178,8 @@ opt("--table-option", action="append", dest="tableopts", default=[],
     help="Add a dialect-specific table option, key=value")
 opt("--coverage", action="callback", callback=_start_coverage,
     help="Dump a full coverage report after running tests")
+opt("--cumulative-coverage", action="callback", callback=_start_cumulative_coverage,
+    help="Like --coverage, but accumlate coverage into the current DB")
 opt("--profile", action="append", dest="profile_targets", default=[],
     help="Enable a named profile target (multiple OK.)")
 opt("--profile-sort", action="store", dest="profile_sort", default=None,
@@ -150,7 +201,7 @@ class _ordered_map(object):
     def __iter__(self):
         for key in self._keys:
             yield self._data[key]
-    
+
 # at one point in refactoring, modules were injecting into the config
 # process.  this could probably just become a list now.
 post_configure = _ordered_map()
@@ -205,9 +256,10 @@ def _engine_pool(options, file_config):
 post_configure['engine_pool'] = _engine_pool
 
 def _create_testing_engine(options, file_config):
-    from testlib import engines
+    from testlib import engines, testing
     global db
     db = engines.testing_engine(db_url, db_opts)
+    testing.db = db
 post_configure['create_engine'] = _create_testing_engine
 
 def _prep_testing_database(options, file_config):
@@ -242,7 +294,7 @@ post_configure['prep_db'] = _prep_testing_database
 
 def _set_table_options(options, file_config):
     import testlib.schema
-    
+
     table_options = testlib.schema.table_options
     for spec in options.tableopts:
         key, value = spec.split('=')
@@ -268,7 +320,7 @@ post_configure['topological'] = _reverse_topological
 
 def _set_profile_targets(options, file_config):
     from testlib import profiling
-    
+
     profile_config = profiling.profile_config
 
     for target in options.profile_targets:
